import type { Express } from "express";
import { createServer, type Server } from "http";
import { storage } from "./storage";
import { insertAuditSchema, insertScanResultSchema, insertUserSchema } from "@shared/schema";
import { z } from "zod";

export async function registerRoutes(app: Express): Promise<Server> {
  // Authentication routes
  app.post("/api/auth/login", async (req, res) => {
    try {
      const { username, password } = req.body;
      const user = await storage.getUserByUsername(username);
      
      if (!user || user.password !== password) {
        return res.status(401).json({ message: "Invalid credentials" });
      }

      res.json({ user: { ...user, password: undefined } });
    } catch (error) {
      res.status(500).json({ message: "Login failed" });
    }
  });

  app.post("/api/auth/register", async (req, res) => {
    try {
      const userData = insertUserSchema.parse(req.body);
      
      const existingUser = await storage.getUserByUsername(userData.username);
      if (existingUser) {
        return res.status(400).json({ message: "Username already exists" });
      }

      const existingEmail = await storage.getUserByEmail(userData.email);
      if (existingEmail) {
        return res.status(400).json({ message: "Email already exists" });
      }

      const user = await storage.createUser(userData);
      res.json({ user: { ...user, password: undefined } });
    } catch (error) {
      res.status(400).json({ message: "Registration failed" });
    }
  });

  // User routes
  app.get("/api/user/:id", async (req, res) => {
    try {
      const id = parseInt(req.params.id);
      const user = await storage.getUser(id);
      
      if (!user) {
        return res.status(404).json({ message: "User not found" });
      }

      res.json({ ...user, password: undefined });
    } catch (error) {
      res.status(500).json({ message: "Failed to get user" });
    }
  });

  // Audit routes
  app.post("/api/audits", async (req, res) => {
    try {
      const auditData = insertAuditSchema.parse(req.body);
      const userId = req.body.userId || 1; // Default to demo user
      
      // Check user credits
      const user = await storage.getUser(userId);
      if (!user || user.credits < 10) {
        return res.status(400).json({ message: "Insufficient credits" });
      }

      const audit = await storage.createAudit(userId, auditData);
      
      // Deduct credits
      await storage.updateUserCredits(userId, user.credits - 10);
      
      res.json(audit);
    } catch (error) {
      res.status(400).json({ message: "Failed to create audit" });
    }
  });

  app.get("/api/audits/user/:userId", async (req, res) => {
    try {
      const userId = parseInt(req.params.userId);
      const audits = await storage.getUserAudits(userId);
      res.json(audits);
    } catch (error) {
      res.status(500).json({ message: "Failed to get audits" });
    }
  });

  app.get("/api/audits/:id", async (req, res) => {
    try {
      const id = parseInt(req.params.id);
      const audit = await storage.getAudit(id);
      
      if (!audit) {
        return res.status(404).json({ message: "Audit not found" });
      }

      res.json(audit);
    } catch (error) {
      res.status(500).json({ message: "Failed to get audit" });
    }
  });

  app.put("/api/audits/:id/progress", async (req, res) => {
    try {
      const id = parseInt(req.params.id);
      const { progress, status } = req.body;
      
      const audit = await storage.updateAuditProgress(id, progress, status);
      if (!audit) {
        return res.status(404).json({ message: "Audit not found" });
      }

      res.json(audit);
    } catch (error) {
      res.status(500).json({ message: "Failed to update audit progress" });
    }
  });

  app.put("/api/audits/:id/results", async (req, res) => {
    try {
      const id = parseInt(req.params.id);
      const { results, securityScore, vulnerabilities, criticalIssues } = req.body;
      
      const audit = await storage.updateAuditResults(id, results, securityScore, vulnerabilities, criticalIssues);
      if (!audit) {
        return res.status(404).json({ message: "Audit not found" });
      }

      res.json(audit);
    } catch (error) {
      res.status(500).json({ message: "Failed to update audit results" });
    }
  });

  // Scanning routes
  app.post("/api/scans/email", async (req, res) => {
    try {
      const { content, userId = 1 } = req.body;
      
      if (!content) {
        return res.status(400).json({ message: "Email content required" });
      }

      // Check user credits
      const user = await storage.getUser(userId);
      if (!user || user.credits < 5) {
        return res.status(400).json({ message: "Insufficient credits" });
      }

      // Simulate AI email analysis
      const threatScore = Math.floor(Math.random() * 30); // 0-30 for demo
      const status = threatScore < 10 ? "clean" : threatScore < 20 ? "suspicious" : "malicious";
      
      const findings = {
        phishing: threatScore > 15,
        malware: threatScore > 20,
        suspiciousLinks: Math.floor(Math.random() * 3),
        attachmentThreats: 0
      };

      const scanResult = await storage.createScanResult(userId, {
        type: "email",
        content,
        threatScore,
        status,
        findings
      });

      // Deduct credits
      await storage.updateUserCredits(userId, user.credits - 5);

      res.json(scanResult);
    } catch (error) {
      res.status(500).json({ message: "Email scan failed" });
    }
  });

  app.post("/api/scans/document", async (req, res) => {
    try {
      const { filename, userId = 1 } = req.body;
      
      if (!filename) {
        return res.status(400).json({ message: "Filename required" });
      }

      // Check user credits
      const user = await storage.getUser(userId);
      if (!user || user.credits < 3) {
        return res.status(400).json({ message: "Insufficient credits" });
      }

      // Simulate document analysis
      const threatScore = Math.floor(Math.random() * 25); // 0-25 for demo
      const status = threatScore < 5 ? "clean" : threatScore < 15 ? "suspicious" : "malicious";
      
      const findings = {
        malware: threatScore > 20,
        macros: Math.floor(Math.random() * 2),
        embeddedObjects: threatScore > 10 ? 1 : 0,
        suspiciousCode: threatScore > 15
      };

      const scanResult = await storage.createScanResult(userId, {
        type: "document",
        filename,
        threatScore,
        status,
        findings
      });

      // Deduct credits
      await storage.updateUserCredits(userId, user.credits - 3);

      res.json(scanResult);
    } catch (error) {
      res.status(500).json({ message: "Document scan failed" });
    }
  });

  app.get("/api/scans/user/:userId", async (req, res) => {
    try {
      const userId = parseInt(req.params.userId);
      const scans = await storage.getUserScanResults(userId);
      res.json(scans);
    } catch (error) {
      res.status(500).json({ message: "Failed to get scan results" });
    }
  });

  // Network scanning endpoint
  app.post("/api/scans/network", async (req, res) => {
    try {
      const { target, scanType, userId = 1 } = req.body;
      
      if (!target) {
        return res.status(400).json({ message: "Target required" });
      }

      // Simulate network scanning
      const results = {
        target,
        scanType,
        openPorts: [22, 80, 443, 3000],
        vulnerabilities: [
          { severity: "high", description: "Outdated SSL certificate" },
          { severity: "medium", description: "Weak cipher suites" },
          { severity: "low", description: "Banner disclosure" }
        ],
        securityScore: Math.floor(Math.random() * 30 + 70),
        recommendations: [
          "Update SSL certificate",
          "Configure stronger cipher suites",
          "Disable banner information disclosure"
        ]
      };

      res.json(results);
    } catch (error) {
      res.status(500).json({ message: "Network scan failed" });
    }
  });

  const httpServer = createServer(app);
  return httpServer;
}
